use strict;
use Getopt::Std;
use Jcode;
use vars qw($VERSION);
$VERSION  = '0.20';
#--------------------------------------
# ϐ錾{R}hCIvV擾
#--------------------------------------
getopts('i:d:t:T:C:P:');
my $hatena_id       = $Getopt::Std::opt_i;
my $bookmark_ymd    = $Getopt::Std::opt_d;
my $bookmark_hhmmss = $Getopt::Std::opt_t;
my $http_proxy      = $Getopt::Std::opt_P;
my $tmpl_filepath   = $Getopt::Std::opt_T;
my $clientCharSet   = ($Getopt::Std::opt_C) ? $Getopt::Std::opt_C : 'sjis';
## K{`FbN
die &showHelp() if($hatena_id eq '');
$bookmark_ymd    =~ s|[\-\/]||g;
$bookmark_hhmmss =~ s|:||g;
$bookmark_hhmmss = substr(qq(${bookmark_hhmmss}000000),0,6);
print Jcode::convert(HSJ::HatenaBookmarkFormatter->new(
  hatena_id  => $hatena_id,
  yyyymmdd   => $bookmark_ymd,
  hhmmss     => $bookmark_hhmmss,
  template   => $tmpl_filepath,
  http_proxy => $http_proxy,
  )->format, $clientCharSet, 'utf8');

sub showHelp(){
  <<  "EOD";
Hatena Bookmark Formatter $VERSION
Copyright(C) DonaDona(KITAO Masato) 2005

Usage: perl $0 [Options]

Required Options:
  -i: Your Hatena Id
Query Options:
  -d: Target Date (YYYYMMDD)
  -t: From Time   (HHMMSS)
Output Options:
  -T: Template FilePath  (encoding: UTF8)
Environment Options:
  -C: Clientside Charset (default: sjis)
  -P: HTTP_PROXY
EOD
}

package HSJ::HatenaBookmarkFormatter;

use LWP::Simple qw($ua get);
use HTML::Template;

#--------------------------------------
# RXgN^
#--------------------------------------
sub new {
  my $class = shift;
  bless {@_}, $class;
}
#--------------------------------------
# `
#--------------------------------------
sub format{
  my $self = shift;
  my @bookmarkList = $self->getBookmarkList();
  return $self->applyTemplate(\@bookmarkList);
}
#--------------------------------------
# ev[g̓Kp
#--------------------------------------
sub applyTemplate{
  my $self    = shift;
  my $refList = shift;
  my $tmpl;
  if($self->{template}){
    $tmpl = HTML::Template->new(filename => $self->{template});
  }
  else{
    $tmpl = HTML::Template->new(filename => 'default.tmpl');
  }
  $tmpl->param(bookmark_list => $refList);
  return $tmpl->output;
}
#--------------------------------------
# ubN}[NnbVXg̎擾
#--------------------------------------
sub getBookmarkList{
  my $self    = shift;
  my $B_URL   = 'http://b.hatena.ne.jp';
  my $param   = '';
  if($self->{yyyymmdd}){
    $param = qq(?date=$self->{yyyymmdd})
  }
  if($self->{http_proxy}){
    $ua->proxy('http', $self->{http_proxy});
  }
  my $res = get(qq(${B_URL}/$self->{hatena_id}/atomfeed${param}))
    or die "Cannot get from ${B_URL}/$self->{hatena_id}/atomfeed${param}";
  my @items = reverse($self->parse_bookmarks($res));
  # WXg̍쐬
  my @genrelist = ();
  foreach my $item (@items){
    if(defined($item->{'tag'})){
      if(grep(/$item->{'tag'}/, @genrelist) == 0){
        push(@genrelist, $item->{'tag'});
      }
    }
  }
  # Wʔz̍쐬
  my @answer;
  foreach my $genre(sort(@genrelist)) {
    my @answerList;
    foreach my $item (@items){
      if(defined($item->{'tag'}) && $item->{'tag'} eq $genre){
        push(@answerList, $item);
      }
    }
    my %hash;
    $hash{'genre'} = $genre;
    $hash{'items'} = \@answerList;
    push(@answer, \%hash);
  }
  @answer;
}
#--------------------------------------
# ubN}[N𕪉AnbVXg
#--------------------------------------
sub parse_bookmarks{
  my $self    = shift;
  my $content = shift;
  my @items;
  #ԖڂtagWƂAtag͖B
  my $REGEXP_BOOKMARKITEM    = '<entry>.*?<title>(.*?)</title>.*?<link rel="related" type="text/html" href="(.*?)".*?/>.*?<issued>([0-9\-]+)T([0-9\:]+)\+09:00</issued>.*?<id>(.*?)</id>.*?<summary.*?>(.*?)</summary>.*?<dc:subject>(.*?)</dc:subject>.*?</entry>';
  while ($content =~ s/$REGEXP_BOOKMARKITEM//is) {
    my ($subject, $link, $date, $time, $id, $comment, $tag) = ($1, $2, $3, $4, $5, $6, $7);
    if($self->{yyyymmdd} ne '' && $self->{hhmmss} ne ''){
      $date =~ s|-||g;
      $time =~ s|:||g;
      if($date eq $self->{yyyymmdd} && $time < $self->{hhmmss}){
        next;
      }
    }
    push(@items, {
      'link'    => $link,
      'id'      => $id,
      'subject' => $subject,
      'tag'     => $tag,
      'comment' => $comment
      });
  }
  @items;
}
1;